EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'DockerImages_GetLocalImages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DockerRepositoryTags_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the Docker image tags for the specified repository.'
GO

IF OBJECT_ID('[DockerImages_GetLocalImages]') IS NOT NULL DROP PROCEDURE [DockerImages_GetLocalImages]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [DockerImages_GetLocalImages]
(
	@FeedIds_Csv VARCHAR(MAX) = NULL,
	@RepositorySearch_Text VARCHAR(MAX) = NULL,

	@Cached_Indicator YNINDICATOR = NULL,
	@OrderBy_Name  VARCHAR(MAX) = NULL, /* Package, Date, Downloads */
	@OrderByDesc_Indicator YNINDICATOR = NULL,
	@Skip_Count INT = NULL,
	@Take_Count INT = NULL
)
AS
BEGIN

SET NOCOUNT ON
	SET @Skip_Count = COALESCE(@Skip_Count, 0)
	SET @Take_Count = COALESCE(@Take_Count, 50)
	/* This is a bit of a disaster due to SQL limitations
	 * [1] using sp_executesql would require us to grant SELECT permissions on the appropriate tables
	 * [2] ROW_NUMBER() OVER's ORDER BY doesn't like constant or CASE'd expressions  (conversion errors)
	 *
	 * The best bet is probably to go with sp_Executesql, but that breaks our model. So, we will just do 
	 * 6 copy/paste statements that are slightly tweaked. Dynamic SQL Version below
	 */

	IF @OrderBy_Name = 'Repository' AND @OrderByDesc_Indicator = 'N'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Repository_Name])
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'Repository' AND @OrderByDesc_Indicator = 'Y'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Repository_Name] DESC)
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
	ELSE IF @OrderBy_Name = 'ImageSize' AND @OrderByDesc_Indicator = 'N'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [DockerImage_Size])
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'ImageSize' AND @OrderByDesc_Indicator = 'Y'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [DockerImage_Size] DESC)
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
	ELSE IF @OrderBy_Name = 'Date' AND @OrderByDesc_Indicator = 'N'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Published_Date])
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'Date' AND @OrderByDesc_Indicator = 'Y'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Published_Date] DESC)
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
	ELSE IF @OrderBy_Name = 'Downloads' AND @OrderByDesc_Indicator = 'N'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Download_Count])
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'Downloads' AND @OrderByDesc_Indicator = 'Y'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Download_Count] DESC)
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
	ELSE
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Repository_Name])
			  FROM [DockerRepositoryTags_Extended]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@RepositorySearch_Text IS NULL OR [Repository_Name] LIKE '%' + @RepositorySearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

END
GO

GRANT EXECUTE ON [DockerImages_GetLocalImages] TO [ProGetUser_Role]
GO