EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ProGetPackages_SetListed',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the value indicating whether the package is cached.'
GO

IF OBJECT_ID('[ProGetPackages_SetListed]') IS NOT NULL DROP PROCEDURE [ProGetPackages_SetListed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ProGetPackages_SetListed]
(
	@Feed_Id INT,
	@Group_Name NVARCHAR(200),
	@Package_Name NVARCHAR(100),
	@Version_Text VARCHAR(255),
    @Listed_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @ProGetPackage_Id INT
		SET @ProGetPackage_Id = (SELECT [ProGetPackage_Id] FROM [ProGetPackages] WHERE [Feed_Id] = @Feed_Id AND [Group_Name] = @Group_Name AND [Package_Name] = @Package_Name)

		UPDATE [ProGetPackageVersions]
		   SET [Listed_Indicator] = @Listed_Indicator
		 WHERE [ProGetPackage_Id] = @ProGetPackage_Id
		   AND [Version_Text] = @Version_Text

		UPDATE [ProGetPackages]
		   SET [Listed_Indicator] = CASE WHEN EXISTS(SELECT * FROM [ProGetPackageVersions] V WHERE V.[ProGetPackage_Id] = @ProGetPackage_Id AND V.[Listed_Indicator] = 'Y') THEN 'Y' ELSE 'N' END
		 WHERE [ProGetPackage_Id] = @ProGetPackage_Id

   		UPDATE [Feeds]
		   SET [FeedState_Number] = COALESCE([FeedState_Number], 0) + 1
		 WHERE [Feed_Id] = @Feed_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ProGetPackages_SetListed] TO [ProGetUser_Role]
GO