EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddPrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Privilege_Id',
    /* Description_Text        */ 'Grants or denies the specified principal the ability to perform a certain role.'
GO

IF OBJECT_ID('[Security_AddPrivilege]') IS NOT NULL DROP PROCEDURE [Security_AddPrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddPrivilege]
(
	@Privilege_Id INT = NULL OUT,
	@UserDirectory_Id INT = NULL,
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@Role_Id INT,
	@Feed_Id INT,
	@FeedGroup_Id INT,
	@PrivilegeType_Code CHAR(1),
	@UseTransaction_Indicator YNINDICATOR = 'Y'
)
AS
BEGIN

SET NOCOUNT ON

	DECLARE @PrivilegeId2 NVARCHAR(MAX)
	IF @UseTransaction_Indicator = 'Y'
	BEGIN
		BEGIN TRY

			BEGIN TRANSACTION

			IF EXISTS(SELECT *
						FROM [Privileges]
						WITH (UPDLOCK)
					WHERE [Principal_Name] = @Principal_Name
						AND ((@UserDirectory_Id IS NULL AND [UserDirectory_Id] IS NULL) OR [UserDirectory_Id] = @UserDirectory_Id)
						AND [PrincipalType_Code] = @PrincipalType_Code
						AND [Role_Id] = @Role_Id
						AND ([Feed_Id] = @Feed_Id OR COALESCE([Feed_Id], @Feed_Id) IS NULL)
						AND ([FeedGroup_Id] = @FeedGroup_Id OR COALESCE([FeedGroup_Id], @FeedGroup_Id) IS NULL)
						AND [PrivilegeType_Code] = @PrivilegeType_Code )
			BEGIN
				ROLLBACK
				RETURN
			END

			INSERT INTO [Privileges]
				([UserDirectory_Id]
				,[Principal_Name]
				,[PrincipalType_Code]
				,[Role_Id]
				,[Feed_Id]
				,[FeedGroup_Id]
				,[PrivilegeType_Code])
			VALUES
				(@UserDirectory_Id
				,@Principal_Name
				,@PrincipalType_Code
				,@Role_Id
				,@Feed_Id
				,@FeedGroup_Id
				,@PrivilegeType_Code)
			SET @Privilege_Id = SCOPE_IDENTITY()

			SET @PrivilegeId2 = CAST(@Privilege_Id AS NVARCHAR(MAX))

			EXEC [Events_RaiseEvent] 'SECNWP', @Feed_Id, 'Privilege_Id', @PrivilegeId2

			COMMIT
		END TRY BEGIN CATCH
			IF XACT_STATE()<>0 ROLLBACK
			EXEC [HandleError]
		END CATCH
	END
	ELSE
	BEGIN
		IF EXISTS(SELECT *
						FROM [Privileges]
						WITH (UPDLOCK)
					WHERE [Principal_Name] = @Principal_Name
						AND ((@UserDirectory_Id IS NULL AND [UserDirectory_Id] IS NULL) OR [UserDirectory_Id] = @UserDirectory_Id)
						AND [PrincipalType_Code] = @PrincipalType_Code
						AND [Role_Id] = @Role_Id
						AND ([Feed_Id] = @Feed_Id OR COALESCE([Feed_Id], @Feed_Id) IS NULL)
						AND ([FeedGroup_Id] = @FeedGroup_Id OR COALESCE([FeedGroup_Id], @FeedGroup_Id) IS NULL)
						AND [PrivilegeType_Code] = @PrivilegeType_Code )
			BEGIN
				RETURN
			END

			INSERT INTO [Privileges]
				([UserDirectory_Id]
				,[Principal_Name]
				,[PrincipalType_Code]
				,[Role_Id]
				,[Feed_Id]
				,[FeedGroup_Id]
				,[PrivilegeType_Code])
			VALUES
				(@UserDirectory_Id
				,@Principal_Name
				,@PrincipalType_Code
				,@Role_Id
				,@Feed_Id
				,@FeedGroup_Id
				,@PrivilegeType_Code)
			SET @Privilege_Id = SCOPE_IDENTITY()

			SET @PrivilegeId2 = CAST(@Privilege_Id AS NVARCHAR(MAX))

			EXEC [Events_RaiseEvent] 'SECNWP', @Feed_Id, 'Privilege_Id', @PrivilegeId2
	END

END
GO

GRANT EXECUTE ON [Security_AddPrivilege] TO [ProGetUser_Role]
GO
